@@ -1,5 +1,5 @@
 /******************************************************************************
-    (c) 2000 Patrick Caulfield                 patrick@debian.org
+    (c) 2002 Patrick Caulfield                 patrick@debian.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,15 +14,31 @@
 
 
 void add_string(unsigned char *packet, int *ptr, 
-		unsigned char *string);
+		const unsigned char *string);
 
 void get_string(unsigned char *packet, int *ptr, 
 		unsigned char *string);
 
-#ifndef USE_OPENPTY
+
+int expand_issue(const char *original, int len,
+		 char *newstring, int maxlen,
+		 const char *servicename);
+
+#ifndef HAVE_OPENPTY
 #define INTERNAL_OPENPTY
 #else
+#ifdef HAVE_PTY_H
 #include <pty.h>
+#endif /* HAVE_PTY_H */
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
+#endif /* HAVE_TERMIOS_H */
+#ifdef HAVE_LIBUTIL_H
+#include <libutil.h>
+#endif /* HAVE_LIBUTIL_H */
+#ifdef HAVE_UTIL_H
+#include <util.h>
+#endif /* HAVE_UTIL_H */
 #endif
 
 // RedHat 5 has a broken openpty()
